# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..types.analytics_query import AnalyticsQuery
from ..core.request_options import RequestOptions
from ..types.analytics_query_result import AnalyticsQueryResult
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AnalyticsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, queries: typing.Sequence[AnalyticsQuery], request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[AnalyticsQueryResult]:
        """
        Parameters
        ----------
        queries : typing.Sequence[AnalyticsQuery]
            This is the list of metric queries you want to perform.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AnalyticsQueryResult]


        Examples
        --------
        from vapi import AnalyticsOperation, AnalyticsQuery, Vapi

        client = Vapi(
            token="YOUR_TOKEN",
        )
        client.analytics.get(
            queries=[
                AnalyticsQuery(
                    name="name",
                    operations=[
                        AnalyticsOperation(
                            operation="sum",
                            column="id",
                        )
                    ],
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "analytics",
            method="POST",
            json={
                "queries": convert_and_respect_annotation_metadata(
                    object_=queries, annotation=typing.Sequence[AnalyticsQuery], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[AnalyticsQueryResult],
                    parse_obj_as(
                        type_=typing.List[AnalyticsQueryResult],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAnalyticsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, queries: typing.Sequence[AnalyticsQuery], request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[AnalyticsQueryResult]:
        """
        Parameters
        ----------
        queries : typing.Sequence[AnalyticsQuery]
            This is the list of metric queries you want to perform.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AnalyticsQueryResult]


        Examples
        --------
        import asyncio

        from vapi import AnalyticsOperation, AnalyticsQuery, AsyncVapi

        client = AsyncVapi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.analytics.get(
                queries=[
                    AnalyticsQuery(
                        name="name",
                        operations=[
                            AnalyticsOperation(
                                operation="sum",
                                column="id",
                            )
                        ],
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "analytics",
            method="POST",
            json={
                "queries": convert_and_respect_annotation_metadata(
                    object_=queries, annotation=typing.Sequence[AnalyticsQuery], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[AnalyticsQueryResult],
                    parse_obj_as(
                        type_=typing.List[AnalyticsQueryResult],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
